@@ -18,6 +18,7 @@
 #include "unicode/smpdtfmt.h"
 #include "unicode/dtfmtsym.h"
 #include "unicode/dtptngen.h"
+#include "unicode/ustring.h"
 #include "cmemory.h"
 #include "loctest.h"
 
@@ -35,6 +36,8 @@ void IntlTestDateTimePatternGeneratorAPI::runIndexedTest( int32_t index, UBool e
         TESTCASE(2, testAllFieldPatterns);
         TESTCASE(3, testStaticGetSkeleton);
         TESTCASE(4, testC);
+        TESTCASE(5, testSkeletonsWithDayPeriods);
+        TESTCASE(6, testGetFieldDisplayNames);
         default: name = ""; break;
     }
 }
@@ -130,7 +133,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("13. Jan."),                            // 05: MMMd
         UnicodeString("13. Januar"),                          // 06: MMMMd
         UnicodeString("Q1 1999"),                             // 07: yQQQ
-        UnicodeString("11:58 nachm."),                        // 08: hhmm
+        UnicodeString("11:58 PM"),                            // 08: hhmm
         UnicodeString("23:58"),                               // 09: HHmm
         UnicodeString("23:58"),                               // 10: jjmm
         UnicodeString("58:59"),                               // 11: mmss
@@ -243,7 +246,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("1/13"),                                            // 04: Md    -> M/d
         CharsToUnicodeString("1\\u670813\\u65E5"),                        // 05: MMMd  ->M\u6708d\u65E5
         CharsToUnicodeString("1\\u670813\\u65E5"),                        // 06: MMMMd  ->M\u6708d\u65E5
-        CharsToUnicodeString("\\u6C11\\u570B88\\u5E741\\u5B63"),          // 07: yQQQ  -> Gy QQQ
+        CharsToUnicodeString("\\u6C11\\u570B88\\u5E74\\u7B2C1\\u5B63"),   // 07: yQQQ  -> Gy QQQ
         CharsToUnicodeString("\\u4E0B\\u534811:58"),                      // 08: hhmm  ->
         UnicodeString("23:58"),                                           // 09: HHmm  ->
         CharsToUnicodeString("\\u4E0B\\u534811:58"),                      // 10: jjmm
@@ -263,7 +266,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         CharsToUnicodeString("13 \\u044F\\u043D\\u0432."),                // 05: MMMd  -> d MMM
         CharsToUnicodeString("13 \\u044F\\u043D\\u0432\\u0430\\u0440\\u044F"), // 06: MMMMd  -> d MMMM
         CharsToUnicodeString("1-\\u0439 \\u043A\\u0432. 1999 \\u0433."),  // 07: yQQQ  -> y QQQ
-        CharsToUnicodeString("11:58 \\u041F\\u041F"),                     // 08: hhmm  -> hh:mm a
+        CharsToUnicodeString("11:58 PM"),                                 // 08: hhmm  -> hh:mm a
         UnicodeString("23:58"),                                           // 09: HHmm  -> HH:mm
         UnicodeString("23:58"),                                           // 10: jjmm  -> HH:mm
         UnicodeString("58:59"),                                           // 11: mmss  -> mm:ss
@@ -355,6 +358,16 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("MMMMMd"),
     };
 
+    const char* testGetSkeletonAndBase[][3] = {
+        // pattern       skeleton    baseSkeleton
+        { "dd-MMM",     "MMMdd",    "MMMd" },
+        { "dd/MMMM/yy", "yyMMMMdd", "yMMMMd" },
+        { "h",          "h",        "h" },
+        { "ah",         "ah",       "ah" },
+        { "aaaah",      "aaaah",    "aaaah" },
+        { "Bh",         "Bh",       "Bh" }
+    };
+
     UnicodeString newDecimal(" "); // space
     UnicodeString newAppendItemName("hrs.");
     UnicodeString newAppendItemFormat("{1} {0}");
@@ -510,34 +523,25 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
     }
 
     // ======== Test getSkeleton and getBaseSkeleton
-    status = U_ZERO_ERROR;
-    pattern = UnicodeString("dd-MMM");
-    UnicodeString expectedSkeleton = UnicodeString("MMMdd");
-    UnicodeString expectedBaseSkeleton = UnicodeString("MMMd");
-    UnicodeString retSkeleton = gen->getSkeleton(pattern, status);
-    if(U_FAILURE(status) || retSkeleton != expectedSkeleton ) {
-         errln("ERROR: Unexpected result from getSkeleton().\n");
-         errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected: ") + expectedSkeleton );
-    }
-    retSkeleton = gen->getBaseSkeleton(pattern, status);
-    if(U_FAILURE(status) || retSkeleton !=  expectedBaseSkeleton) {
-         errln("ERROR: Unexpected result from getBaseSkeleton().\n");
-         errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected:")+ expectedBaseSkeleton);
+    
+    int32_t i, count = UPRV_LENGTHOF(testGetSkeletonAndBase);
+    for (i = 0; i < count; i++) {
+        status = U_ZERO_ERROR;
+        pattern                            = UnicodeString(testGetSkeletonAndBase[i][0]);
+        UnicodeString expectedSkeleton     = UnicodeString(testGetSkeletonAndBase[i][1]);
+        UnicodeString expectedBaseSkeleton = UnicodeString(testGetSkeletonAndBase[i][2]);
+        UnicodeString retSkeleton = gen->getSkeleton(pattern, status);
+		if(U_FAILURE(status) || retSkeleton != expectedSkeleton ) {
+			 errln("ERROR: Unexpected result from getSkeleton().\n");
+			 errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected: ") + expectedSkeleton );
+		}
+		retSkeleton = gen->getBaseSkeleton(pattern, status);
+		if(U_FAILURE(status) || retSkeleton !=  expectedBaseSkeleton) {
+			 errln("ERROR: Unexpected result from getBaseSkeleton().\n");
+			 errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected:")+ expectedBaseSkeleton);
+		}
     }
 
-    pattern = UnicodeString("dd/MMMM/yy");
-    expectedSkeleton = UnicodeString("yyMMMMdd");
-    expectedBaseSkeleton = UnicodeString("yMMMMd");
-    retSkeleton = gen->getSkeleton(pattern, status);
-    if(U_FAILURE(status) || retSkeleton != expectedSkeleton ) {
-         errln("ERROR: Unexpected result from getSkeleton().\n");
-         errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected: ") + expectedSkeleton );
-    }
-    retSkeleton = gen->getBaseSkeleton(pattern, status);
-    if(U_FAILURE(status) || retSkeleton !=  expectedBaseSkeleton) {
-         errln("ERROR: Unexpected result from getBaseSkeleton().\n");
-         errln(UnicodeString(" Got: ") + retSkeleton + UnicodeString(" Expected:")+ expectedBaseSkeleton);
-    }
     delete format;
     delete zone;
     delete gen;
@@ -710,7 +714,6 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         return;
     }
     UChar newChar;
-    int32_t i;
     for (i=0; i<10; ++i) {
         UnicodeString randomSkeleton;
         int32_t len = rand() % 20;
@@ -770,7 +773,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
     }
     UnicodeString returnPattern, *ptrSkeleton;
     ptrSkeletonEnum->reset(status);
-    int32_t count=ptrSkeletonEnum->count(status);
+    count=ptrSkeletonEnum->count(status);
     for (i=0; i<count; ++i) {
         ptrSkeleton = (UnicodeString *)ptrSkeletonEnum->snext(status);
         returnPattern = test->getPatternForSkeleton(*ptrSkeleton);
@@ -976,13 +979,17 @@ void IntlTestDateTimePatternGeneratorAPI::testAllFieldPatterns(/*char *par*/)
         { 'e',  {1,2,3,4,5,6},  "Eec"  }, // local day of week
         { 'c',  {1,2,3,4,5,6},  "Eec"  }, // standalone local day of week
         // day period
-    //  { 'a',  {1,0},          "a"    }, // am or pm   // not clear this one is supposed to work (it doesn't)
+        { 'a',  {1,2,3,4,5,0},  "a"    }, // am or pm
+        { 'b',  {1,2,3,4,5,0},  "b"    }, // dayPeriod AM/PM/noon
+        { 'B',  {1,2,3,4,5,0},  "B"    }, // dayPeriod ranges
         // hour
         { 'h',  {1,2,0},        "hK"   }, // 12 (1-12)
         { 'H',  {1,2,0},        "Hk"   }, // 24 (0-23)
         { 'K',  {1,2,0},        "hK"   }, // 12 (0-11)
         { 'k',  {1,2,0},        "Hk"   }, // 24 (1-24)
         { 'j',  {1,2,0},        "hHKk" }, // locale default
+        { 'J',  {1,2,0},        "hHKk" }, // locale default, without any dayPeriod
+        { 'C',  {1,2,0},        "hHKk" }, // locale allowed first entry, possibly with b or B
         // minute
         { 'm',  {1,2,0},        "m"    }, // x
         // second & fractions
@@ -1084,25 +1091,39 @@ void IntlTestDateTimePatternGeneratorAPI::testStaticGetSkeleton(/*char *par*/)
 }
 
 void IntlTestDateTimePatternGeneratorAPI::testC() {
-    UErrorCode status = U_ZERO_ERROR;
-    const int32_t numLocales = 6;
-
-    const char* tests[numLocales][3] = {
-            {"zh", "Cm", "Bh:mm"},
-            {"de", "Cm", "HH:mm"},
-            {"en", "Cm", "h:mm a"},
-            {"en-BN", "Cm", "h:mm b"},
-            {"gu-IN", "Cm", "h:mm B"},
-            {"und-IN", "Cm", "h:mm a"},
+    const char* tests[][3] = {
+            // These may change with actual data for Bhmm/bhmm skeletons
+            {"zh",     "Cm",      "Bh:mm"},
+            {"zh",     "CCm",     "Bhh:mm"},
+            {"zh",     "CCCm",    "BBBBh:mm"},
+            {"zh",     "CCCCm",   "BBBBhh:mm"},
+            {"zh",     "CCCCCm",  "BBBBBh:mm"},
+            {"zh",     "CCCCCCm", "BBBBBhh:mm"},
+            {"de",     "Cm",      "HH:mm"},
+            {"de",     "CCm",     "HH:mm"},
+            {"de",     "CCCm",    "HH:mm"},
+            {"de",     "CCCCm",   "HH:mm"},
+            {"en",     "Cm",      "h:mm a"},
+            {"en",     "CCm",     "hh:mm a"},
+            {"en",     "CCCm",    "h:mm aaaa"},
+            {"en",     "CCCCm",   "hh:mm aaaa"},
+            {"en",     "CCCCCm",  "h:mm aaaaa"},
+            {"en",     "CCCCCCm", "hh:mm aaaaa"},
+            {"en-BN",  "Cm",      "h:mm b"},
+            {"gu-IN",  "Cm",      "h:mm B"},
+            {"und-IN", "Cm",      "h:mm a"}
     };
 
-    for (int32_t i = 0; i < numLocales; ++i) {
+    UErrorCode status = U_ZERO_ERROR;
+    int32_t numTests = UPRV_LENGTHOF(tests);
+    for (int32_t i = 0; i < numTests; ++i) {
         DateTimePatternGenerator *gen = DateTimePatternGenerator::createInstance(Locale(tests[i][0]), status);
         if (gen == NULL) {
             dataerrln("FAIL: DateTimePatternGenerator::createInstance failed for %s", tests[i][0]);
             return;
         }
-        UnicodeString pattern = gen->getBestPattern(tests[i][1], status);
+        UDateTimePatternMatchOptions options = UDATPG_MATCH_HOUR_FIELD_LENGTH;
+        UnicodeString pattern = gen->getBestPattern(tests[i][1], options, status);
         UnicodeString expectedPattern = tests[i][2];
 
         char message[100] = "\0";
@@ -1114,4 +1135,128 @@ void IntlTestDateTimePatternGeneratorAPI::testC() {
     }
 }
 
+enum { kCharBufMax = 31 };
+void IntlTestDateTimePatternGeneratorAPI::testSkeletonsWithDayPeriods() {
+    const char * patterns[] = {
+        // since icu4c getEmptyInstance does not call addCanonicalItems (unlike J), set these here:
+        "a",    // should get internal skeleton a
+        "H",    // should get internalskeleton H
+        "m",    // should get internalskeleton m
+        "s",    // should get internalskeleton s
+        // patterns from which to construct sample data for a locale
+        //"H",  // should get internalskeleton H
+        "h a",  // should get internalskeleton ah
+        "B h",  // should get internalskeleton Bh
+    };
+    const char* testItems[][2] = {
+        // sample requested skeletons and results
+        // skel     pattern
+        { "H",      "H"},
+        { "HH",     "HH"},
+        { "aH",     "H"},
+        { "aHH",    "HH"},
+        { "BH",     "H"},
+        { "BHH",    "HH"},
+        { "BBBBH",  "H"},
+        { "h",      "h a"},
+        { "hh",     "hh a"},
+        { "ah",     "h a"},
+        { "ahh",    "hh a"},
+        { "aaaah",  "h aaaa"},
+        { "aaaahh", "hh aaaa"},
+        { "bh",     "h b"},
+        { "bhh",    "hh b"},
+        { "bbbbh",  "h bbbb"},
+        { "Bh",     "B h"},
+        { "Bhh",    "B hh"},
+        { "BBBBh",  "BBBB h"},
+        { "BBBBhh", "BBBB hh"},
+        { "a",      "a"},
+        { "aaaaa",  "aaaaa"},
+        { "b",      "b"},
+        { "bbbb",   "bbbb"},
+        { "B",      "B"},
+        { "BBBB",  "BBBB"},
+    };
+    UErrorCode status = U_ZERO_ERROR;
+    DateTimePatternGenerator *gen = DateTimePatternGenerator::createEmptyInstance(status);
+    if (U_FAILURE(status)) {
+        errln("ERROR: createEmptyInstance fails, status: %s", u_errorName(status));
+    } else {
+        int32_t i, len = UPRV_LENGTHOF(patterns);
+        for (i = 0; i < len; i++) {
+            UnicodeString conflictingPattern;
+            (void)gen->addPattern(UnicodeString(patterns[i]), TRUE, conflictingPattern, status);
+            if (U_FAILURE(status)) {
+                errln("ERROR: addPattern %s fail, status: %s", patterns[i], u_errorName(status));
+                break;
+            }
+        }
+        if (U_SUCCESS(status)) {
+            len = UPRV_LENGTHOF(testItems);
+            for (i = 0; i < len; i++) {
+                status = U_ZERO_ERROR;
+                UDateTimePatternMatchOptions options = UDATPG_MATCH_HOUR_FIELD_LENGTH;
+                UnicodeString result = gen->getBestPattern(UnicodeString(testItems[i][0]), options, status);
+                if (U_FAILURE(status)) {
+                    errln("ERROR: getBestPattern %s fail, status: %s", testItems[i][0], u_errorName(status));
+                } else if (result != UnicodeString(testItems[i][1])) {
+                    char charResult[kCharBufMax+1];
+                    result.extract(0, result.length(), charResult, kCharBufMax);
+                    charResult[kCharBufMax] = 0; // ensure termination
+                    errln("ERROR: getBestPattern %s, expected %s, got %s", testItems[i][0], testItems[i][1], charResult);
+                }
+            }
+        }
+    }
+    delete gen;
+}
+
+typedef struct FieldDisplayNameData {
+    const char *            locale;
+    UDateTimePatternField   field;
+    UDateTimePGDisplayWidth width;
+    const char *            expected; // can have escapes such as \\u00E0
+} FieldDisplayNameData;
+enum { kFieldDisplayNameMax = 32 };
+
+void IntlTestDateTimePatternGeneratorAPI::testGetFieldDisplayNames() {
+    const FieldDisplayNameData testData[] = {
+        /*loc      field                              width               expectedName */
+        { "de",    UDATPG_QUARTER_FIELD,              UDATPG_WIDE,        "Quartal" },
+        { "de",    UDATPG_QUARTER_FIELD,              UDATPG_ABBREVIATED, "Quart." },
+        { "de",    UDATPG_QUARTER_FIELD,              UDATPG_NARROW,      "Q" },
+        { "en",    UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_WIDE,        "weekday of the month" },
+        { "en",    UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_ABBREVIATED, "wkday. of mo." },
+        { "en",    UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_NARROW,      "wkday. of mo." }, // fallback
+        { "en_GB", UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_WIDE,        "weekday of the month" },
+        { "en_GB", UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_ABBREVIATED, "wkday of mo" }, // override
+        { "en_GB", UDATPG_DAY_OF_WEEK_IN_MONTH_FIELD, UDATPG_NARROW,      "wkday of mo" },
+        { "it",    UDATPG_SECOND_FIELD,               UDATPG_WIDE,        "secondo" },
+        { "it",    UDATPG_SECOND_FIELD,               UDATPG_ABBREVIATED, "s" },
+        { "it",    UDATPG_SECOND_FIELD,               UDATPG_NARROW,      "s" },
+    };
+
+    int count = UPRV_LENGTHOF(testData);
+    const FieldDisplayNameData * testDataPtr = testData;
+    for (; count-- > 0; ++testDataPtr) {
+        UErrorCode status = U_ZERO_ERROR;
+        Locale locale(testDataPtr->locale);
+        DateTimePatternGenerator * dtpg = DateTimePatternGenerator::createInstance(locale, status);
+        if (U_FAILURE(status)) {
+            dataerrln("FAIL: DateTimePatternGenerator::createInstance failed for locale %s", testDataPtr->locale);
+        } else {
+            UChar expName[kFieldDisplayNameMax+1];
+            u_unescape(testDataPtr->expected, expName, kFieldDisplayNameMax);
+            expName[kFieldDisplayNameMax] = 0; // ensure 0 termination
+            UnicodeString getName = dtpg->getFieldDisplayName(testDataPtr->field, testDataPtr->width);
+            if (getName.compare(expName, u_strlen(expName)) != 0) {
+                errln("ERROR: locale %s field %d width %d, expected %s\n",
+                      testDataPtr->locale, testDataPtr->field, testDataPtr->width, testDataPtr->expected);
+            }
+            delete dtpg;
+        }
+    }
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
